package cn.com.ty.lift.business.project.controller;

import cn.com.ty.lift.business.framework.util.MessageUtils;
import cn.com.ty.lift.business.project.dao.entity.ProjectLiftRelevance;
import cn.com.ty.lift.business.library.dao.entity.model.BatchUpdateLiftRequest;
import cn.com.ty.lift.business.project.service.ProjectLiftRelevanceService;
import cn.com.ty.lift.business.project.dao.entity.model.ProjectLiftRelevanceRequest;
import cn.com.ty.lift.business.project.dao.entity.model.ProjectRelevanceResponse;
import cn.com.ty.lift.common.constants.ApiConstants;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;

/**
 * @author bieao
 * @date 2019/12/26
 * @description 项目电梯关联控制层
 */
@RestController
@RequestMapping("project/relevance")
public class ProjectRelevanceController {

    @Resource
    private ProjectLiftRelevanceService projectRelevanceService;

    /**
     * @param request updateLiftList 项目id列表
     * @return 1.成功, 0.失败, 消息描述
     * @description 电梯批量锁定、解锁
     * @date 2019/11/27 10:03 AM
     */
    @PostMapping("lockOrUnlock")
    public RestResponse lockOrUnlock(@RequestBody BatchUpdateLiftRequest request) {
        return projectRelevanceService.lockOrUnlock(request);
    }

    /**
     * @param projectLiftRelevance 关联表主键id
     * @return 1.成功, 0.失败, 消息描述
     * @description 逻辑删除电梯
     * @date 2019/11/27 10:03 AM
     */
    @PostMapping("logicDelete")
    public RestResponse logicDelete(@RequestBody ProjectLiftRelevance projectLiftRelevance) {
        return projectRelevanceService.logicDelete(projectLiftRelevance);
    }


    /**
     * @param request 用户id
     * @return 1.成功, 0.失败, 消息描述
     * @description 查询用户下的关联项目和电梯信息
     * @date 2019/12/26 4:27 PM
     */
    @PostMapping("list")
    public RestResponse list(@RequestBody ProjectLiftRelevanceRequest request) {
        IPage<ProjectRelevanceResponse> page = projectRelevanceService.list(request);
        if (ObjectUtil.isEmpty(page.getRecords())) {
            return RestResponse.success();
        }
        return RestResponse.success(page, ApiConstants.RESULT_SUCCESS, MessageUtils.get("msg.query.success"));
    }
}
