package cn.com.ty.lift.business.project.service;

import cn.com.ty.lift.business.framework.util.MessageUtils;
import cn.com.ty.lift.business.library.dao.entity.Lift;
import cn.com.ty.lift.business.project.dao.entity.ProjectLiftRelevance;
import cn.com.ty.lift.business.library.dao.entity.model.BatchUpdateLiftRequest;
import cn.com.ty.lift.business.library.dao.entity.model.LiftExtensionRequest;
import cn.com.ty.lift.business.project.dao.entity.model.ProjectLiftRelevanceRequest;
import cn.com.ty.lift.business.project.dao.mapper.ProjectLiftRelevanceMapper;
import cn.com.ty.lift.business.project.dao.entity.Project;
import cn.com.ty.lift.business.project.dao.entity.model.ProjectRelevanceResponse;
import cn.com.ty.lift.business.project.dao.mapper.ProjectMapper;
import cn.com.ty.lift.common.constants.ApiConstants;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;

/**
 * @author bieao
 * @date 2019/12/5
 * @description 项目电梯关联业务层
 */
@Service
public class ProjectLiftRelevanceService extends ServiceImpl<ProjectLiftRelevanceMapper, ProjectLiftRelevance> {

    @Resource
    private ProjectMapper projectMapper;

    @Resource
    private ProjectLiftRelevanceMapper projectLiftRelevanceMapper;
    /**
     * @param entry 项目关联数据
     * @return RestResponse 状态码和返回消息
     * @description 逻辑删除电梯
     * @date 2019/12/5 2:16 PM
     */
    public RestResponse logicDelete(ProjectLiftRelevance entry) {
        boolean result = updateById(entry);
        if (!result) {
            return RestResponse.fail(MessageUtils.get("msg.modify.fail"));
        }
        return RestResponse.success(null, ApiConstants.RESULT_SUCCESS, MessageUtils.get("msg.modify.success"));
    }

    /**
     * @param request 电梯扩展请求
     * @param id        平台企业电梯主键id
     * @param workerId  维保工id
     * @return projectLiftEntry
     * @description 新增项目电梯关联数据
     * @date 2019/12/5 2:39 PM
     */
    public boolean save(LiftExtensionRequest request, long id, long workerId) {
        Lift lift = request.getLift();
        //新增项目电梯关联表
        ProjectLiftRelevance projectLiftEntry = new ProjectLiftRelevance();
        projectLiftEntry.setProjectId(request.getProjectId());
        projectLiftEntry.setLiftId(lift.getId());
        projectLiftEntry.setRelevanceId(id);
        projectLiftEntry.setWorkerId(workerId);
        projectLiftEntry.setCompanyId(request.getMtCompanyId());
        return save(projectLiftEntry);
    }

    /**
     * @param request
     * @return RestResponse 状态码和返回消息
     * @description 项目锁定、解锁,锁定该项目下的电梯
     * @date 2019/12/5 11:00 AM
     */
    @Transactional
    public RestResponse lockOrUnlock(BatchUpdateLiftRequest request) {
        String source = request.getSource();
        String projectId = request.getProjectId();
        switch (source) {
            //锁定电梯
            case "0":
                List<ProjectLiftRelevance> list = request.getUpdateLiftList();
                if (ObjectUtil.isNotEmpty(list)) list.forEach(entry -> entry.setLiftLocked(true));
                boolean result = updateBatchById(list);
                if (!result) {
                    return RestResponse.fail(MessageUtils.get("msg.modify.fail"));
                }
                break;
            //锁定项目
            case "1":
                Project project = new Project();
                project.setId(Long.parseLong(projectId));
                project.setLocked(true);
                int ret = projectMapper.updateById(project);
                if (ret < 0){
                    return RestResponse.fail(MessageUtils.get("msg.modify.fail"));
                }
                //根据项目id和公司id查询项目下电梯
                List<ProjectLiftRelevance> relevanceList = projectLiftRelevanceMapper.findIdList(request);
                relevanceList.forEach(relevance -> relevance.setLiftLocked(true));
                boolean response = updateBatchById(relevanceList);
                if (!response){
                    return RestResponse.fail(MessageUtils.get("msg.modify.fail"));
                }
                break;
            default:
                break;
        }
        return RestResponse.success(null, ApiConstants.RESULT_SUCCESS, MessageUtils.get("msg.modify.success"));
    }

    /**
     * @param request 用户id
     * @return 1.成功, 0.失败, 消息描述
     * @description 查询用户下的关联项目和电梯信息
     * @date 2019/12/26 4:27 PM
     */
    public IPage<ProjectRelevanceResponse> list(ProjectLiftRelevanceRequest request){
        IPage<ProjectRelevanceResponse> page = new Page<>(request.getPageNum(), request.getPageSize());
        return projectLiftRelevanceMapper.selectListByUserId(page, request);
    }
}
